@@ -9,10 +9,10 @@
 static void match_calc_mtx( double **mtx, double *match, char **s1, char **s2, int i1, int lgth2 ) 
 {
 	char *seq2 = s2[0];
-	double *doubleptr = mtx[(int)s1[0][i1]];
+	double *doubleptr = mtx[(unsigned char)s1[0][i1]];
 
 	while( lgth2-- )
-		*match++ = doubleptr[(int)*seq2++];
+		*match++ = doubleptr[(unsigned char)*seq2++];
 }
 #else
 static void match_calc( double *match, char **s1, char **s2, int i1, int lgth2 )
@@ -366,7 +366,7 @@ double G__align11( double **n_dynamicmtx, char **seq1, char **seq2, int alloclen
 
 		doublework = AllocateFloatMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
 		intwork = AllocateIntMtx( nalphabets, MAX( ll1, ll2 )+2 ); 
-		amino_dynamicmtx = AllocateDoubleMtx( 0x80, 0x80 );
+		amino_dynamicmtx = AllocateDoubleMtx( 0x100, 0x100 );
 
 #if DEBUG
 		fprintf( stderr, "succeeded\n" );
@@ -376,7 +376,7 @@ double G__align11( double **n_dynamicmtx, char **seq1, char **seq2, int alloclen
 		orlgth2 = ll2 - 100;
 	}
     for( i=0; i<nalphabets; i++) for( j=0; j<nalphabets; j++ )
-		amino_dynamicmtx[(int)amino[i]][(int)amino[j]] = (double)n_dynamicmtx[i][j];
+		amino_dynamicmtx[(unsigned char)amino[i]][(unsigned char)amino[j]] = (double)n_dynamicmtx[i][j];
 
 
 	mseq1[0] = mseq[0];
