@@ -219,6 +219,19 @@ void FreeFloatMtx( double **mtx )
 	free( mtx );
 }
 
+int *AllocateIntVecLarge( unsigned long long ll1 )
+{
+	int *vec;
+
+	vec = (int *)calloc( ll1, sizeof( int ) );
+	if( vec == NULL )
+	{	
+		fprintf( stderr, "Allocation error( %lld int vec )\n", ll1 );
+		exit( 1 );
+	}
+	return( vec );
+}	
+
 int *AllocateIntVec( int ll1 )
 {
 	int *vec;
@@ -294,6 +307,29 @@ int **AllocateIntMtx( int ll1, int ll2 )
 	return( mtx );
 }
 
+int **AllocateIntMtxLarge( unsigned long long ll1, unsigned long long ll2 )
+{
+	unsigned long long i;
+	int **mtx;
+
+	mtx = (int **)calloc( ll1+1, sizeof( int * ) );
+	if( !mtx )
+	{
+		fprintf( stderr, "Allocation error( %lld x %lld int mtx )\n", ll1, ll2 );
+		exit( 1 );
+	}
+	if( ll2 )
+	{
+		for( i=0; i<ll1; i++ ) mtx[i] = AllocateIntVecLarge( ll2 );
+	}
+	else
+	{
+		for( i=0; i<ll1; i++ ) mtx[i] = NULL;
+	}
+	mtx[ll1] = NULL;
+	return( mtx );
+}
+
 /*
 void FreeIntMtx( int **mtx )
 {
