@@ -20,9 +20,11 @@ double **AlocateFloatTri( int );
 void FreeFloatTri( double ** );
 
 int *AllocateIntVec( int );
+int *AllocateIntVecLarge( unsigned long long );
 void FreeIntVec( int * );
 
 int **AllocateIntMtx( int, int );
+int **AllocateIntMtxLarge( unsigned long long, unsigned long long );
 void FreeIntMtx( int ** );
 
 char ***AllocateCharCub( int, int, int );
