@@ -100,173 +100,45 @@ static void part_imp_match_out_vead_tate_gapmap( double *imp, int j1, int lgth1,
 #endif
 }
 
-void part_imp_match_init_strict( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1_kozo, double *eff2_kozo, LocalHom ***localhom, int forscore )
+#if 1
+void part_imp_match_init_strict( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, double *eff1_kozo, double *eff2_kozo, LocalHom ***localhom, char *swaplist, int forscore, int *orinum1, int *orinum2 )
 {
-	int i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	double effij, effijx, effij_kozo; 
-	char *pt, *pt1, *pt2;
-	LocalHom *tmpptr;
+//	int i, j, k1, k2, tmpint, start1, start2, end1, end2;
+//	double effij;
+//	double effij_kozo;
+//	double effijx;
+//	char *pt, *pt1, *pt2;
+//	static TLS char *nocount1 = NULL;
+//	static TLS char *nocount2 = NULL;
+//	LocalHom *tmpptr;
 
 	if( seq1 == NULL )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
 		impmtx = NULL;
+//		if( nocount1 ) free( nocount1 );
+//		nocount1 = NULL;
+//		if( nocount2 ) free( nocount2 );
+//		nocount2 = NULL;
+		
 		return;
 	}
 
-	if( impalloclen <= lgth1 + 2 || impalloclen <= lgth2 + 2 )
+	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
+//		if( nocount1 ) free( nocount1 );
+//		if( nocount2 ) free( nocount2 );
 		impalloclen = MAX( lgth1, lgth2 ) + 2;
-		impmtx = AllocateFloatMtx( impalloclen+100, impalloclen+100 );
+		impmtx = AllocateFloatMtx( impalloclen, impalloclen );
+//		nocount1 = AllocateCharVec( impalloclen );
+//		nocount2 = AllocateCharVec( impalloclen );
 	}
 
-
-#if 0
-	fprintf( stderr, "eff1 in _init_strict = \n" );
-	for( i=0; i<clus1; i++ )
-		fprintf( stderr, "eff1[] = %f\n", eff1[i] );
-	for( i=0; i<clus2; i++ )
-		fprintf( stderr, "eff2[] = %f\n", eff2[i] );
-#endif
-
-	for( i=0; i<lgth1; i++ ) for( j=0; j<lgth2; j++ )
-		impmtx[i][j] = 0.0;
-	effijx = 1.0 * fastathreshold;
-	for( i=0; i<clus1; i++ )
-	{
-		for( j=0; j<clus2; j++ )
-		{
-			effij = eff1[i] * eff2[j] * effijx;
-			effij_kozo = eff1_kozo[i] * eff2_kozo[j] * effijx;
-			tmpptr = localhom[i][j];
-			while( tmpptr )
-			{
-//				fprintf( stderr, "start1 = %d\n", tmpptr->start1 );
-//				fprintf( stderr, "end1   = %d\n", tmpptr->end1   );
-//				fprintf( stderr, "i = %d, seq1 = \n%s\n", i, seq1[i] );
-//				fprintf( stderr, "j = %d, seq2 = \n%s\n", j, seq2[j] );
-				pt = seq1[i];
-				tmpint = -1;
-				while( *pt != 0 )
-				{
-					if( *pt++ != '-' ) tmpint++;
-					if( tmpint == tmpptr->start1 ) break;
-				}
-				start1 = (int)( pt - seq1[i] ) - 1;
-	
-				if( tmpptr->start1 == tmpptr->end1 ) end1 = start1;
-				else
-				{
-#if MACHIGAI
-					while( *pt != 0 )
-					{
-						if( tmpint == tmpptr->end1 ) break;
-						if( *pt++ != '-' ) tmpint++;
-					}
-					end1 = (int)( pt - seq1[i] ) - 1;
-#else
-					while( *pt != 0 )
-					{
-//						fprintf( stderr, "tmpint = %d, end1 = %d pos = %d\n", tmpint, tmpptr->end1, pt-seq1[i] );
-						if( *pt++ != '-' ) tmpint++;
-						if( tmpint == tmpptr->end1 ) break;
-					}
-					end1 = (int)( pt - seq1[i] ) - 1;
-#endif
-				}
-	
-				pt = seq2[j];
-				tmpint = -1;
-				while( *pt != 0 )
-				{
-					if( *pt++ != '-' ) tmpint++;
-					if( tmpint == tmpptr->start2 ) break;
-				}
-				start2 = (int)( pt - seq2[j] ) - 1;
-				if( tmpptr->start2 == tmpptr->end2 ) end2 = start2;
-				else
-				{
-#if MACHIGAI
-					while( *pt != 0 )
-					{
-						if( tmpint == tmpptr->end2 ) break;
-						if( *pt++ != '-' ) tmpint++;
-					}
-					end2 = (int)( pt - seq2[j] ) - 1;
-#else
-					while( *pt != 0 )
-					{
-						if( *pt++ != '-' ) tmpint++;
-						if( tmpint == tmpptr->end2 ) break;
-					}
-					end2 = (int)( pt - seq2[j] ) - 1;
-#endif
-				}
-//				fprintf( stderr, "start1 = %d (%c), end1 = %d (%c), start2 = %d (%c), end2 = %d (%c)\n", start1, seq1[i][start1], end1, seq1[i][end1], start2, seq2[j][start2], end2, seq2[j][end2] );
-//				fprintf( stderr, "step 0\n" );
-				if( end1 - start1 != end2 - start2 )
-				{
-//					fprintf( stderr, "CHUUI!!, start1 = %d, end1 = %d, start2 = %d, end2 = %d\n", start1, end1, start2, end2 );
-				}
-
-				k1 = start1; k2 = start2;
-				pt1 = seq1[i] + k1;
-				pt2 = seq2[j] + k2;
-				while( *pt1 && *pt2 )
-				{
-					if( *pt1 != '-' && *pt2 != '-' )
-					{
-// ŤߤŤˤʤ褦դƲ
-//						impmtx[k1][k2] += tmpptr->wimportance * fastathreshold;
-//						impmtx[k1][k2] += tmpptr->importance * effij;
-//						impmtx[k1][k2] += tmpptr->fimportance * effij;
-						if( tmpptr->korh == 'k' )
-							impmtx[k1][k2] += tmpptr->fimportance * effij_kozo;
-						else
-							impmtx[k1][k2] += tmpptr->fimportance * effij;
-//						fprintf( stderr, "k1=%d, k2=%d, impalloclen=%d\n", k1, k2, impalloclen );
-//						fprintf( stderr, "mark, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k1++; k2++;
-						pt1++; pt2++;
-					}
-					else if( *pt1 != '-' && *pt2 == '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k2++; pt2++;
-					}
-					else if( *pt1 == '-' && *pt2 != '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k1++; pt1++;
-					}
-					else if( *pt1 == '-' && *pt2 == '-' )
-					{
-//						fprintf( stderr, "skip, %d (%c) - %d (%c) \n", k1, *pt1, k2, *pt2 );
-						k1++; pt1++;
-						k2++; pt2++;
-					}
-					if( k1 > end1 || k2 > end2 ) break;
-				}
-				tmpptr = tmpptr->next;
-			}
-		}
-	}
-#if 0
-	fprintf( stderr, "impmtx = \n" );
-	for( k2=0; k2<lgth2; k2++ )
-		fprintf( stderr, "%6.3f ", (double)k2 );
-	fprintf( stderr, "\n" );
-	for( k1=0; k1<lgth1; k1++ )
-	{
-		fprintf( stderr, "%d", k1 );
-		for( k2=0; k2<lgth2; k2++ )
-			fprintf( stderr, "%2.1f ", impmtx[k1][k2] );
-		fprintf( stderr, "\n" );
-	}
-	exit( 1 );
-#endif
+	fillimp( impmtx, imp, clus1, clus2, lgth1, lgth2, seq1, seq2, eff1, eff2, eff1_kozo, eff2_kozo, localhom, swaplist, forscore, orinum1, orinum2 );
 }
+#else
+#endif
 
 
 void part_imp_rna( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1, double *eff2, RNApair ***grouprna1, RNApair ***grouprna2, int *gapmap1, int *gapmap2, RNApair *additionalpair )
@@ -275,157 +147,6 @@ void part_imp_rna( int nseq1, int nseq2, char **seq1, char **seq2, double *eff1,
 }
 
 
-void part_imp_match_init( double *imp, int clus1, int clus2, int lgth1, int lgth2, char **seq1, char **seq2, double *eff1, double *eff2, LocalHom ***localhom )
-{
-	int dif, i, j, k1, k2, tmpint, start1, start2, end1, end2;
-	static TLS int impalloclen = 0;
-	char *pt;
-	static TLS char *nocount1 = NULL;
-	static TLS char *nocount2 = NULL;
-
-	if( impalloclen < lgth1 || impalloclen < lgth2 )
-	{
-		if( impmtx ) FreeFloatMtx( impmtx );
-		if( nocount1 ) free( nocount1 );
-		if( nocount2 ) free( nocount2 );
-		impalloclen = MAX( lgth1, lgth2 ) + 2;
-		impmtx = AllocateFloatMtx( impalloclen, impalloclen );
-		nocount1 = AllocateCharVec( impalloclen );
-		nocount2 = AllocateCharVec( impalloclen );
-		impalloclen -= 2;
-	}
-
-	for( i=0; i<lgth1; i++ )
-	{
-		for( j=0; j<clus1; j++ )
-			if( seq1[j][i] == '-' ) break;
-		if( j != clus1 ) nocount1[i] = 1; 
-		else			 nocount1[i] = 0;
-	}
-	for( i=0; i<lgth2; i++ )
-	{
-		for( j=0; j<clus2; j++ )
-			if( seq2[j][i] == '-' ) break;
-		if( j != clus2 ) nocount2[i] = 1;
-		else			 nocount2[i] = 0;
-	}
-
-#if 0
-fprintf( stderr, "nocount2 =\n" );
-for( i = 0; i<impalloclen; i++ )
-{
-	fprintf( stderr, "nocount2[%d] = %d (%c)\n", i, nocount2[i], seq2[0][i] );
-}
-#endif
-
-	for( i=0; i<lgth1; i++ ) for( j=0; j<lgth2; j++ )
-		impmtx[i][j] = 0.0;
-	for( i=0; i<clus1; i++ )
-	{
-		fprintf( stderr, "i = %d, seq1 = %s\n", i, seq1[i] );
-		for( j=0; j<clus2; j++ )
-		{
-			fprintf( stderr, "start1 = %d\n", localhom[i][j]->start1 );
-			fprintf( stderr, "end1   = %d\n", localhom[i][j]->end1   );
-			fprintf( stderr, "j = %d, seq2 = %s\n", j, seq2[j] );
-			pt = seq1[i];
-			tmpint = -1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->start1 ) break;
-			}
-			start1 = pt - seq1[i] - 1;
-
-			while( *pt != 0 )
-			{
-//				fprintf( stderr, "tmpint = %d, end1 = %d pos = %d\n", tmpint, localhom[i][j].end1, pt-seq1[i] );
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->end1 ) break;
-			}
-			end1 = pt - seq1[i] - 1;
-
-			pt = seq2[j];
-			tmpint = -1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->start2 ) break;
-			}
-			start2 = pt - seq2[j] - 1;
-			while( *pt != 0 )
-			{
-				if( *pt++ != '-' ) tmpint++;
-				if( tmpint == localhom[i][j]->end2 ) break;
-			}
-			end2 = pt - seq2[j] - 1;
-//			fprintf( stderr, "start1 = %d, end1 = %d, start2 = %d, end2 = %d\n", start1, end1, start2, end2 );
-			k1 = start1;
-			k2 = start2;
-			fprintf( stderr, "step 0\n" );
-			while( k1 <= end1 && k2 <= end2 )
-			{
-#if 0
-				if( !nocount1[k1] && !nocount2[k2] )
-					impmtx[k1][k2] += localhom[i][j].wimportance * eff1[i] * eff2[j];
-				k1++; k2++;
-#else
-				if( !nocount1[k1] && !nocount2[k2] )
-					impmtx[k1][k2] += localhom[i][j]->wimportance * eff1[i] * eff2[j];
-				k1++; k2++;
-#endif
-			}
-
-			dif = ( end1 - start1 ) - ( end2 - start2 );
-			fprintf( stderr, "dif = %d\n", dif );
-			if( dif > 0 )
-			{
-				do
-				{
-					fprintf( stderr, "dif = %d\n", dif );
-					k1 = start1;
-					k2 = start2 - dif;
-					while( k1 <= end1 && k2 <= end2 )
-					{
-						if( 0 <= k2 && start2 <= k2 && !nocount1[k1] && !nocount2[k2] )
-							impmtx[k1][k2] = localhom[i][j]->wimportance * eff1[i] * eff2[j];
-						k1++; k2++;
-					}
-				}
-				while( dif-- );
-			}
-			else
-			{
-				do
-				{
-					k1 = start1 + dif;
-					k2 = start2;
-					while( k1 <= end1 )
-					{
-						if( k1 >= 0 && k1 >= start1 && !nocount1[k1] && !nocount2[k2] )
-							impmtx[k1][k2] = localhom[i][j]->wimportance * eff1[i] * eff2[j];
-						k1++; k2++;
-					}
-				}
-				while( dif++ );
-			}
-		}
-	}
-#if 0
-	fprintf( stderr, "impmtx = \n" );
-	for( k2=0; k2<lgth2; k2++ )
-		fprintf( stderr, "%6.3f ", (double)k2 );
-	fprintf( stderr, "\n" );
-	for( k1=0; k1<lgth1; k1++ )
-	{
-		fprintf( stderr, "%d", k1 );
-		for( k2=0; k2<lgth2; k2++ )
-			fprintf( stderr, "%6.3f ", impmtx[k1][k2] );
-		fprintf( stderr, "\n" );
-	}
-	exit( 1 );
-#endif
-}
 
 
 static void match_calc( double *match, double **cpmx1, double **cpmx2, int i1, int lgth2, double **doublework, int **intwork, int initialize )
@@ -1031,7 +752,7 @@ double partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int i
 			orlgth1 = 0;
 			orlgth2 = 0;
 
-			part_imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+			part_imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
 
 			free( mseq1 );
 			free( mseq2 );
@@ -1720,7 +1441,7 @@ double partA__align_variousdist( int **which, double ***matrices, double **n_dyn
 			orlgth1 = 0;
 			orlgth2 = 0;
 
-			part_imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+			part_imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0, NULL, NULL );
 
 			free( mseq1 );
 			free( mseq2 );
@@ -1782,7 +1503,7 @@ double partA__align_variousdist( int **which, double ***matrices, double **n_dyn
 		}
 	}
 	for( c=0; c<maxdistclass; c++ ) if( nmask[c] ) break;
-	if( c<maxdistclass ) reporterr( "Found a complex grouping. Use an ultrametric tree to make the calculation faster.\n" );
+	if( c<maxdistclass ) reporterr( "Found a complex grouping. This step may be a bit slow.\n" );
 
 	lgth1 = strlen( seq1[0] );
 	lgth2 = strlen( seq2[0] );
