@@ -21,19 +21,36 @@ static int smoothing;
 static int specifictarget;
 static int callpairlocalalign;
 static int outputhat23;
+static int compacttree = 0;
 
-#ifdef enablemultithread
 typedef struct _jobtable
 {
     int i;  
     int j;  
 } Jobtable;
 
+typedef struct _msacompactdistmtxthread_arg // single thread demo tsukau
+{
+	int njob;
+	int thread_no;
+	int *selfscore;
+	double **partmtx;
+	char **seq;
+	int **skiptable;
+	double *mindist;
+	int *mindistfrom;
+ 	int *jobpospt;
+#ifdef enablemultithread
+	pthread_mutex_t *mutex;
+#endif
+} msacompactdistmtxthread_arg_t;
+
+#ifdef enablemultithread
 typedef struct _distancematrixthread_arg
 {
 	int njob;
 	int thread_no;
-	double *selfscore;
+	int *selfscore;
 	double **iscore;
 	char **seq;
 	int **skiptable;
@@ -550,7 +567,90 @@ static void *distancematrixthread2( void *arg )
 }
 #endif
 
+static double preferenceval( int ori, int pos, int max ) // for debug
+{
+	pos -= ori;
+	if( pos < 0 ) pos += max;
+	return( 0.00000000000001 * pos );
+}
+
+static void *msacompactdisthalfmtxthread( void *arg ) // enablemultithread == 0 demo tsukau
+{
+	msacompactdistmtxthread_arg_t *targ = (msacompactdistmtxthread_arg_t *)arg;
+	int njob = targ->njob;
+	int thread_no = targ->thread_no;
+	int *selfscore = targ->selfscore;
+	double **partmtx = targ->partmtx;
+	char **seq = targ->seq;
+	int **skiptable = targ->skiptable;
+	double *mindist = targ->mindist;
+	int *mindistfrom = targ->mindistfrom;
+ 	int *jobpospt = targ->jobpospt;
+	double tmpdist, preference, tmpdistx, tmpdisty;
+	int i, j;
+
+	while( 1 )
+	{
 #ifdef enablemultithread
+		if( nthread ) 
+		{
+			pthread_mutex_lock( targ->mutex );
+			i = *jobpospt;
+			if( i == njob-1 )
+			{
+				pthread_mutex_unlock( targ->mutex );
+				return( NULL );
+			}
+			*jobpospt = i+1;
+			pthread_mutex_unlock( targ->mutex );
+		}
+		else
+#endif
+		{
+			i = *jobpospt;
+			if( i == njob-1 )
+			{
+				return( NULL );
+			}
+			*jobpospt = i+1;
+		}
+
+		if( i % 100 == 0 ) 
+		{
+			if( nthread )
+				fprintf( stderr, "\r% 5d / %d (thread %4d)", i, njob, thread_no );
+			else
+				fprintf( stderr, "\r% 5d / %d", i, njob );
+		}
+
+		for( j=i+1; j<njob; j++ ) 
+		{
+			tmpdist = distcompact_msa( seq[i], seq[j], skiptable[i], skiptable[j], selfscore[i], selfscore[j] ); // osoikedo,
+
+			preference = preferenceval( i, j, njob );
+			tmpdistx = tmpdist + preference;
+			if( tmpdistx < mindist[i] )
+			{
+				mindist[i] = tmpdistx;
+				mindistfrom[i] = j;
+			}
+
+			preference = preferenceval( j, i, njob );
+			tmpdisty = tmpdist + preference;
+			if( tmpdisty < mindist[j] )
+			{
+				mindist[j] = tmpdisty;
+				mindistfrom[j] = i;
+			}
+			if( partmtx[i] ) partmtx[i][j] = tmpdist;
+			if( partmtx[j] ) partmtx[j][i] = tmpdist;
+		}
+	}
+}
+
+
+#ifdef enablemultithread
+#if 0
 static void *distancematrixthread( void *arg ) // v7.2 ijou deha tsukawanaihazu
 {
 	distancematrixthread_arg_t *targ = (distancematrixthread_arg_t *)arg;
@@ -598,7 +698,51 @@ static void *distancematrixthread( void *arg ) // v7.2 ijou deha tsukawanaihazu
 		if( iscore[i][j-i] > 10 ) iscore[i][j-i] = 10.0; // 2015/Mar/17
 	}
 }
+#else
+static void *distancematrixthread( void *arg ) // v7.2 ijou deha tsukawanaihazu
+{
+	distancematrixthread_arg_t *targ = (distancematrixthread_arg_t *)arg;
+	int njob = targ->njob;
+	int thread_no = targ->thread_no;
+	int *selfscore = targ->selfscore;
+	double **iscore = targ->iscore;
+	char **seq = targ->seq;
+	int **skiptable = targ->skiptable;
+	Jobtable *jobpospt = targ->jobpospt;
 
+	int ssi, ssj, bunbo;
+	int i, j;
+
+	while( 1 )
+	{
+		pthread_mutex_lock( targ->mutex );
+		i = jobpospt->i; // (jobpospt-i)++ dato, shuuryou hantei no mae ni ++ surunode, tomaranakunaru.
+
+
+		if( i == njob-1 )
+		{
+			pthread_mutex_unlock( targ->mutex );
+			return( NULL );
+		}
+		jobpospt->i += 1;
+		pthread_mutex_unlock( targ->mutex );
+		if( i % 100 == 0 ) fprintf( stderr, "\r% 5d / %d (thread %4d)", i, njob, thread_no );
+
+		ssi = selfscore[i];
+		for( j=i+1; j<njob; j++ )
+		{
+			ssj = selfscore[j];
+			bunbo = MIN( ssi, ssj );
+			if( bunbo == 0 )
+				iscore[i][j-i] = 2.0; // 2013/Oct/17
+			else
+//			iscore[i][j-i] = ( 1.0 - naivepairscore11( seq[i], seq[j], penalty_dist ) / bunbo ) * 2.0; // 2013/Oct/17
+				iscore[i][j-i] = ( 1.0 - naivepairscorefast( seq[i], seq[j], skiptable[i], skiptable[j], penalty_dist ) / bunbo ) * 2.0; // 2014/Aug/15 fast
+			if( iscore[i][j-i] > 10.0 ) iscore[i][j-i] = 10.0; // 2015/Mar/17
+		}
+	}
+}
+#endif
 static void *treebasethread( void *arg )
 {
 	treebasethread_arg_t *targ = (treebasethread_arg_t *)arg;
@@ -1483,30 +1627,77 @@ static void WriteOptions( FILE *fp )
 	fflush( fp );
 }
 	 
+static double **preparepartmtx( int nseq )
+{
+	int i;
+	double **val;
+	double size;
+
+	val = (double **)calloc( nseq, sizeof( double *) );;
+	size = 0;
+
+#if 0
+	if( compacttree == 1 )
+	{
+		for( i=0; i<nseq; i++ )
+		{
+			size += (double)sizeof( double ) * nseq;
+			if( size > maxdistmtxsize ) 
+			{
+				reporterr( "\n\nThe size of full distance matrix is estimated to exceed %.2fGB.\n", maxdistmtxsize / 1000 / 1000 /1000 );
+				reporterr( "Will try the calculation using a %d x %d matrix.\n", nseq, i );
+				reporterr( "This calculation will be slow due to the limited RAM space.\n", i, nseq );
+				reporterr( "To avoid the slowdown, please try '--initialramusage xGB' (x>>%.2f),\n", maxdistmtxsize / 1000 / 1000 /1000 );
+				reporterr( "if larger RAM space is available.\n" );
+				reporterr( "Note that xGB is NOT the upper limit of RAM usage.\n" );
+				reporterr( "Two to three times larger space may be used for building a guide tree.\n" );
+				reporterr( "Memory usage of the MSA stage depends on similarity of input sequences.\n\n" );
+//				reporterr( "If the RAM is small, try '--initialramusage xGB' with a smaller x value.\n" );
+				reporterr( "The '--memsavetree' option uses smaller RAM space.\n" );
+				reporterr( "If tree-like relationship can be ignored, try '--pileup' or '--randomchain'.\n\n" );
+				reporterr( "The result of --initialramusage xGB is almost identical to the default, except for rounding differences.\n" );
+
+				reporterr( "In the cases of --memsavetree, --pileup and --randomchain, the result differs from the default.\n\n" );
+				break;
+			}
+			val[i] = (double *)calloc( nseq, sizeof( double ) );
+		}
+		if( i == nseq ) reporterr( "The full matrix will be used.\n" );
+
+		for( ;i<nseq; i++ ) val[i] = NULL; // nen no tame
+	}
+	else
+#endif
+	{
+		for( i=0; i<nseq; i++ ) val[i] = NULL; // nen no tame
+	}
+	return( val );
+}
+
 
 int main( int argc, char *argv[] )
 {
-	static int  *nlen;	
-	static double *selfscore;
+	static int  *nlen = NULL;	
+	static int *selfscore = NULL;
 	int nogaplen;
-	static char **name, **seq;
-	static char **mseq1, **mseq2;
-	static char **bseq;
-	static double **iscore, **iscore_kozo;
+	static char **name = NULL, **seq = NULL;
+	static char **mseq1 = NULL, **mseq2 = NULL;
+	static char **bseq = NULL;
+	static double **iscore = NULL, **iscore_kozo = NULL;
 	int **skiptable;
-	static double *eff, *eff_kozo, *eff_kozo_mapped = NULL;
+	static double *eff = NULL, *eff_kozo = NULL, *eff_kozo_mapped = NULL;
 	int i, j, ien, ik, jk;
-	static int ***topol, ***topol_kozo;
+	static int ***topol = NULL, ***topol_kozo = NULL;
 	static int *addmem;
-	static Treedep *dep;
-	static double **len, **len_kozo;
-	FILE *prep;
-	FILE *infp;
-	FILE *orderfp;
-	FILE *hat2p;
+	static Treedep *dep = NULL;
+	static double **len = NULL, **len_kozo = NULL;
+	FILE *prep = NULL;
+	FILE *infp = NULL;
+	FILE *orderfp = NULL;
+	FILE *hat2p = NULL;
 	double unweightedspscore;
 	int alignmentlength;
-	char *mergeoralign;
+	char *mergeoralign = NULL;
 	int foundthebranch;
 	int nsubalignments, maxmem;
 	int **subtable;
@@ -1517,6 +1708,11 @@ int main( int argc, char *argv[] )
 	char **addbk = NULL;
 	int **deletelist = NULL;
 	FILE *dlf = NULL;
+// for compacttree
+	int *mindistfrom = NULL;
+	double *mindist = NULL;
+	double **partmtx = NULL;
+// for compacttree
 
 	char c;
 	int alloclen;
@@ -1524,10 +1720,10 @@ int main( int argc, char *argv[] )
 	LocalHom *tmpptr;
 	RNApair ***singlerna = NULL;
 	double ssi, ssj, bunbo;
-	static char *kozoarivec;
+	static char *kozoarivec = NULL;
 	int nkozo;
 	int ntarget;
-	int *targetmap, *targetmapr;
+	int *targetmap = NULL, *targetmapr = NULL;
 	int ilim, jst, jj;
 	int pac, tac;
 	char **pav, **tav;
@@ -1585,11 +1781,10 @@ int main( int argc, char *argv[] )
 
 	name = AllocateCharMtx( njob, B+1 );
 	nlen = AllocateIntVec( njob );
-	selfscore = AllocateFloatVec( njob );
+	selfscore = AllocateIntVec( njob );
 
 	topol = AllocateIntCub( njob, 2, 0 );
 	len = AllocateFloatMtx( njob, 2 );
-	iscore = AllocateFloatHalfMtx( njob );
 	eff = AllocateDoubleVec( njob );
 	kozoarivec = AllocateCharVec( njob );
 
@@ -1598,6 +1793,9 @@ int main( int argc, char *argv[] )
 	dep = (Treedep *)calloc( njob, sizeof( Treedep ) );
 	if( nadd ) addmem = AllocateIntVec( nadd+1 );
 
+
+	if( tbutree ) iscore = AllocateFloatHalfMtx( njob ); // tbutree=0 no toki aln kara mtx wo keisan, compacttree dehanaitoki nomi iscore shiyou.
+
 	ndeleted = 0;
 
 #if 0
@@ -1638,7 +1836,8 @@ int main( int argc, char *argv[] )
 		reporterr( "targetmapr[%d] = %d\n", i, targetmapr[i] );
 #endif
 
-	if( constraint && !noalign ) // 2016mar15 noalign tsuika
+//	if( constraint && !noalign ) // 2016mar15 noalign tsuika
+	if( constraint ) // 2016Jul31 noalign no toki no shori (l=0.0) ha mafft.tmpl ni idou
 	{
 
 		ilim = njob;
@@ -1763,8 +1962,6 @@ int main( int argc, char *argv[] )
 			fclose( prep );
 			fprintf( stderr, "\ndone.\n" );
 		}
-		free( tav );
-		free( pav );
 
 
 
@@ -1805,8 +2002,6 @@ int main( int argc, char *argv[] )
 		{
 			pairlocalalign( njob, nlenmax, name, seq, iscore, NULL, pac, pav );
 			arguments( tac, tav, NULL, NULL, NULL, NULL ); // anzen no tame
-			free( tav );
-			free( pav );
 			callpairlocalalign = 1; // wakarinikui.
 			if( fastathreshold < 0.0001 ) constraint = 0; 
 			fprintf( stderr, "blosum %d / kimura 200\n", nblosum );
@@ -1815,6 +2010,7 @@ int main( int argc, char *argv[] )
 		}
 		if( distout || outputhat23 )
 		{
+			reporterr( "\nwriting hat2 (1)\n" );
 			prep = fopen( "hat2", "w" );
 			WriteFloatHat2_pointer_halfmtx( prep, njob, name, iscore );
 			fclose( prep );
@@ -1822,10 +2018,11 @@ int main( int argc, char *argv[] )
 	}
 
 
-
-
+	free( tav );
+	free( pav );
 	constants( njob, seq );
 
+
 #if 0
 	fprintf( stderr, "params = %d, %d, %d\n", penalty, penalty_ex, offset );
 #endif
@@ -1836,6 +2033,17 @@ int main( int argc, char *argv[] )
 
 	WriteOptions( trap_g );
 
+	if( distout && !treeout && noalign )  // 2016Jul31. Free ha mada fukanzen.
+	{
+		writeData_pointer( prep_g, njob, name, nlen, seq );
+		fprintf( stderr, "\n" ); 
+		SHOWVERSION;
+		goto chudan;
+	}
+
+
+
+
 	c = seqcheck( seq );
 	if( c )
 	{
@@ -1845,6 +2053,24 @@ int main( int argc, char *argv[] )
 
 //	writePre( njob, name, nlen, seq, 0 );
 
+	if( treein )
+	{
+		int dumx, dumy;
+		double dumz;
+		treein = check_guidetreefile( &dumx, &dumy, &dumz );
+		if( treein == 'C' )
+		{
+			compacttree = 2;
+			treein = 0;
+			use_fft = 0; // kankeinai?
+		}
+
+		// else treein = 1 no mama
+	}
+
+	reporterr( "treein = %d\n", treein );
+	reporterr( "compacttree = %d\n", compacttree );
+
 	if( nadd && keeplength )
 	{
 		originalgaps = (char *)calloc( nlenmax+1, sizeof( char) );
@@ -1885,11 +2111,129 @@ int main( int argc, char *argv[] )
 	}
 	else
 	{
-		if( tbutree == 0 )
+		if( tbutree == 0 && compacttree ) // compacttree no toki ha treein ha 0 de uwagaki sarete iru.
 		{
-			reporterr( "Bug.  This function should not be used in versions >7.2.  Please email kazutaka.katoh@aist.go.jp\n" );
-			fflush( stderr );
-			exit( 1 );
+			iscore = NULL;// tsukawanai
+			reporterr( "Making a compact tree from msa, step 1.. \n" );
+			skiptable = AllocateIntMtx( njob, 0 );
+			makeskiptable( njob, skiptable, seq ); // allocate suru.
+			mindistfrom = (int *)calloc( njob, sizeof( int ) );
+			mindist = (double *)calloc( njob, sizeof( double ) );
+			partmtx = preparepartmtx( njob );
+
+			for( i=0; i<njob; i++ ) // disttbfast deha kokoniha nakatta.
+			{
+//				selfscore[i] = naivepairscore11( seq[i], seq[i], penalty_dist );
+				selfscore[i] = (int)naivepairscorefast( seq[i], seq[i], skiptable[i], skiptable[i], penalty_dist );
+//				fprintf( stderr, "penalty = %d\n", penalty );
+//				fprintf( stderr, "penalty_dist = %d\n", penalty_dist );
+			}
+#ifdef enablemultithread
+			if( nthread > 0 )
+			{
+				msacompactdistmtxthread_arg_t *targ;
+				int jobpos;
+				pthread_t *handle;
+				pthread_mutex_t mutex;
+				double **mindistthread;
+				int **mindistfromthread;
+
+				mindistthread = AllocateDoubleMtx( nthread, njob );
+				mindistfromthread = AllocateIntMtx( nthread, njob );
+				targ = calloc( nthread, sizeof( msacompactdistmtxthread_arg_t ) );
+				handle = calloc( nthread, sizeof( pthread_t ) );
+				pthread_mutex_init( &mutex, NULL );
+				jobpos = 0;
+
+				for( i=0; i<nthread; i++ )
+				{
+					for( j=0; j<njob; j++ )
+					{
+						mindistthread[i][j] = 999.9;
+						mindistfromthread[i][j] = -1;
+					}
+					targ[i].thread_no = i;
+					targ[i].njob = njob;
+					targ[i].selfscore = selfscore;
+					targ[i].partmtx = partmtx;
+					targ[i].seq = seq;
+					targ[i].skiptable = skiptable;
+					targ[i].jobpospt = &jobpos;
+					targ[i].mindistfrom = mindistfromthread[i];
+					targ[i].mindist = mindistthread[i];
+					targ[i].mutex = &mutex;
+	
+					pthread_create( handle+i, NULL, msacompactdisthalfmtxthread, (void *)(targ+i) );
+				}
+	
+				for( i=0; i<nthread; i++ ) pthread_join( handle[i], NULL );
+				pthread_mutex_destroy( &mutex );
+
+				for( i=0; i<njob; i++ )
+				{
+					mindist[i] = 999.9;
+					mindistfrom[i] = -1;
+					for( j=0; j<nthread; j++ )
+					{
+						if( mindistthread[j][i] < mindist[i] )
+						{
+							mindist[i] = mindistthread[j][i];
+							mindistfrom[i] = mindistfromthread[j][i];
+						}
+					}
+				}
+				for( i=0; i<njob; i++ ) mindist[i] -= preferenceval( i, mindistfrom[i], njob ); // for debug
+
+				free( handle );
+				free( targ );
+				FreeDoubleMtx( mindistthread );
+				FreeIntMtx( mindistfromthread );
+			}
+			else
+#endif
+			{
+				msacompactdistmtxthread_arg_t *targ;
+				int jobpos;
+				jobpos = 0;
+				targ = calloc( 1, sizeof( msacompactdistmtxthread_arg_t ) );
+
+				{
+					for( j=0; j<njob; j++ )
+					{
+						mindist[j] = 999.9;
+						mindistfrom[j] = -1;
+					}
+					targ[0].thread_no = 0;
+					targ[0].njob = njob;
+					targ[0].selfscore = selfscore;
+					targ[0].partmtx = partmtx;
+					targ[0].seq = seq;
+					targ[0].skiptable = skiptable;
+					targ[0].jobpospt = &jobpos;
+					targ[0].mindistfrom = mindistfrom;
+					targ[0].mindist = mindist;
+	
+					msacompactdisthalfmtxthread( targ );
+//					msacompactdistmtxthread( targ );
+				}
+				free( targ );
+				for( i=0; i<njob; i++ ) mindist[i] -= preferenceval( i, mindistfrom[i], njob ); // for debug
+			}
+//			free( selfscore ); selfscore = NULL; // mada tsukau
+//			FreeCharMtx( bseq ); bseq = NULL; // mada tsukau
+//			if( skiptable) FreeIntMtx( skiptable ); skiptable = NULL;
+
+//			for( i=0; i<njob; i++ ) printf( "mindist[%d] = %f\n", i, mindist[i] );
+//			exit( 1 );
+			reporterr( "\rdone.                                          \n" );
+		}
+		else if( tbutree == 0 && compacttree == 0 )
+		{
+			reporterr( "Making a distance matrix from msa .. \n" );
+//			reporterr( "Bug.  This function should not be used in versions >=7.2.  Please email kazutaka.katoh@aist.go.jp\n" );
+//			fflush( stderr );
+//			exit( 1 );
+			iscore = AllocateFloatHalfMtx( njob );  // tbutree == 0 no baai ha allocate sareteinainode
 
 			for( i=1; i<njob; i++ ) 
 			{
@@ -1906,7 +2250,7 @@ int main( int argc, char *argv[] )
 			for( i=0; i<njob; i++ ) 
 			{
 //				selfscore[i] = naivepairscore11( seq[i], seq[i], penalty_dist );
-				selfscore[i] = naivepairscorefast( seq[i], seq[i], skiptable[i], skiptable[i], penalty_dist );
+				selfscore[i] = (int)naivepairscorefast( seq[i], seq[i], skiptable[i], skiptable[i], penalty_dist );
 //				fprintf( stderr, "penalty = %d\n", penalty );
 //				fprintf( stderr, "penalty_dist = %d\n", penalty_dist );
 			}
@@ -1983,6 +2327,8 @@ int main( int argc, char *argv[] )
 //			fprintf( stderr, "\ndone.\n\n" );
 			FreeIntMtx( skiptable );
 //			fflush( stderr );
+			reporterr( "\rdone.                                           \n" );
+
 		}
 		else
 		{
@@ -2030,6 +2376,7 @@ int main( int argc, char *argv[] )
 
 				if( distout ) // callpairlocalalign == 1 no toki ha ue de shorizumi.
 				{
+					reporterr( "\nwriting hat2 (2)\n" );
 					hat2p = fopen( "hat2", "w" );
 					WriteFloatHat2_pointer_halfmtx( hat2p, njob, name, iscore );
 					fclose( hat2p );
@@ -2057,7 +2404,7 @@ int main( int argc, char *argv[] )
 			}
 		}
 
-		fprintf( stderr, "Constructing a UPGMA tree ... " );
+//		fprintf( stderr, "Constructing a UPGMA tree ... " );
 		fflush( stderr );
 		if( topin )
 		{
@@ -2069,14 +2416,27 @@ int main( int argc, char *argv[] )
 		}
 		else if( subalignment ) // merge error no tame
 		{
+			fprintf( stderr, "Constructing a UPGMA tree ... " );
 			fixed_supg_double_realloc_nobk_halfmtx_treeout_constrained( njob, iscore, topol, len, name, nlen, dep, nsubalignments, subtable, 1 );
 		}
+		else if( tbutree == 0 && compacttree ) // tbutree != 0 no toki (aln->mtx) ha, 6merdistance -> disttbfast.c; dp distance -> muzukashii
+		{
+			reporterr(       "Constructing a tree ... " );
+			compacttree_memsaveselectable( njob, partmtx, mindistfrom, mindist, NULL, selfscore, seq, skiptable, topol, len, name, NULL, dep, treeout, compacttree, 1 );
+			if( mindistfrom ) free( mindistfrom ); mindistfrom = NULL;
+			if( mindist ) free( mindist );; mindist = NULL;
+//			if( selfscore ) free( selfscore ); selfscore = NULL; // matomete free
+			if( skiptable) FreeIntMtx( skiptable ); skiptable = NULL; // nikaime dake
+			free( partmtx );
+		}
 		else if( treeout ) // merge error no tame
 		{
+			fprintf( stderr, "Constructing a UPGMA tree ... " );
 			fixed_musclesupg_double_realloc_nobk_halfmtx_treeout( njob, iscore, topol, len, name, nlen, dep, 1 );
 		}
 		else
 		{
+			fprintf( stderr, "Constructing a UPGMA tree ... " );
 			fixed_musclesupg_double_realloc_nobk_halfmtx( njob, iscore, topol, len, dep, 1, 1 );
 		}
 //		else 
@@ -2116,7 +2476,7 @@ int main( int argc, char *argv[] )
 		writeData_pointer( prep_g, njob, name, nlen, seq );
 		fprintf( stderr, "\n" ); 
 		SHOWVERSION;
-		return( 0 );
+		goto chudan; // 2016Jul31
 	}
 
 //	countnode( njob, topol, node0 );
@@ -2172,7 +2532,7 @@ int main( int argc, char *argv[] )
 		}
 	}
 
-	FreeFloatHalfMtx( iscore, njob );
+	if( iscore ) FreeFloatHalfMtx( iscore, njob ); iscore = NULL;
 	FreeFloatMtx( len );
 
 	alloclen = nlenmax*2+1; //chuui!
@@ -2684,4 +3044,41 @@ int main( int argc, char *argv[] )
 	if( nadd ) free( addmem );
 
 	return( 0 );
+
+chudan:
+	if( seq ) FreeCharMtx( seq ); seq = NULL;
+	if( mseq1 ) free( mseq1 ); mseq1 = NULL;
+	if( mseq2 ) free( mseq2 ); mseq2 = NULL;
+
+	if( name ) FreeCharMtx( name ); name = NULL;
+	if( nlen ) free( nlen ); nlen = NULL;
+	if( selfscore ) free( selfscore ); selfscore = NULL;
+	if( mergeoralign ) free( mergeoralign ); mergeoralign = NULL;
+
+	if( localhomtable )
+	{
+		reporterr( "freeing localhomtable\n" );
+		if( specifictarget )
+			FreeLocalHomTable_part( localhomtable, ntarget, njob );
+		else
+			FreeLocalHomTable_half( localhomtable, njob );
+	} 
+	localhomtable = NULL;
+	if( targetmap ) free( targetmap ); targetmap = NULL;
+	if( targetmapr ) free( targetmapr ); targetmapr = NULL;
+
+	if( kozoarivec ) free( kozoarivec ); kozoarivec = NULL;
+
+
+	if( topol ) FreeIntCub( topol ); topol = NULL;
+	if( len ) FreeFloatMtx( len ); len = NULL;
+	if( iscore ) FreeFloatHalfMtx( iscore, njob ); iscore = NULL;
+	if( eff ) free( eff ); eff = NULL;
+	if( dep ) free( dep ); dep = NULL;
+
+	freeconstants();
+	closeFiles();
+	FreeCommonIP();
+	return( 0 );
+
 }
